/*
 Calisto (c) 1998-1999 Peter Howkins, Matthew Howkins, Simon Howkins

 $Id: colours.c,v 1.3 1999/12/11 23:08:53 peter Exp $

 $Log: colours.c,v $
 Revision 1.3  1999/12/11 23:08:53  peter
 minor code tidying, including const's, static's and removing one function

 Revision 1.2  1999/12/06 21:19:13  peter
 Moved #define's and typedef into .c file

 Revision 1.1  1999/11/23 22:23:40  peter
 Initial revision


 */
static char rcsid[] = "$Id: colours.c,v 1.3 1999/12/11 23:08:53 peter Exp $";

/* Emacs mode style select: -*- C++ -*- */
/**************************************************************************\
 * Filename:             colours.c                                        *
 * Description:          ANSI Colour handling module.                     *
 * Author:               Mo McKinlay [Cirrus].                            *
 * Date:   [0.0.1]       4-Dec-1997 - Original.                           *
 *         [0.0.2]       5-Dec-1997 - Changes:                            *
 *                       - Added colours.h header file.                   *
 *                       - Added c..printf() functions.                   *
 **************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "colours.h"

#define DEF_FG -1
#define DEF_BG -1

#define cfBold      0x01
#define cfItalics   0x02
#define cfFlash     0x04
#define cfUnderline 0x08
#define cfReverse   0x10

#define MAX_BUF_LEN 10240

typedef struct {
  unsigned char flags;
  int fgColour;
  int bgColour;
} ColRec;

static void writeColRec(char *dest, const ColRec *colRec) {
  char tmpBuf[5];
  int fg, bg;
  *dest=0;
  strncat(dest, "\033[0m", MAX_BUF_LEN-1);
  if((*colRec).flags&cfFlash) strncat(dest, "\033[5m", MAX_BUF_LEN-1);
  if((*colRec).flags&cfBold) strncat(dest, "\033[1m", MAX_BUF_LEN-1);
  /* Unimplemented: cfUnderline & cfItalics */
  if((*colRec).flags&cfUnderline) strncat(dest, "\033[4m", MAX_BUF_LEN-1);
  if((*colRec).flags&cfItalics) strncat(dest, "\033[4m", MAX_BUF_LEN-1);
  if((*colRec).flags&cfReverse) {
    fg=(*colRec).bgColour;
    bg=(*colRec).fgColour;
  } else {
    fg=(*colRec).fgColour;
    bg=(*colRec).bgColour;
  };
  if(fg!=-1) {	
    while(fg>15) fg-=16;
    if(fg>7) {
      strncat(dest, "\033[1m", MAX_BUF_LEN-1);
      sprintf(tmpBuf, "\033[%dm", 22+fg);
      strncat(dest, tmpBuf, MAX_BUF_LEN-1);
    } else {
      sprintf(tmpBuf, "\033[%dm", 30+fg);
      strncat(dest, tmpBuf, MAX_BUF_LEN-1);
    };
  };
  if(bg!=-1) {
    while(bg>7) bg-=8;
    sprintf(tmpBuf, "\033[%dm", 40+bg);
    strncat(dest, tmpBuf, MAX_BUF_LEN-1);
  };
}

static void setColour(char *dest, ColRec *colRec, char colour) {
   char tempBuf[MAX_BUF_LEN];
   if(!(*colRec).flags&&cfReverse) {
      (*colRec).fgColour=colour;
   } else {
      (*colRec).bgColour=colour;
   };
   writeColRec(tempBuf, colRec);
   strncat(dest, tempBuf, MAX_BUF_LEN-1);
}

static void setBackground(char *dest, ColRec *colRec, char colour) {
   char tempBuf[MAX_BUF_LEN];
   if(!(*colRec).flags&&cfReverse) {
      (*colRec).bgColour=colour;
   } else {
      (*colRec).fgColour=colour;
   };
   writeColRec(tempBuf, colRec);
   strncat(dest, tempBuf, MAX_BUF_LEN-1);
}

static void addColourCode(char *dest, ColRec *colRec, char code) {
   char tempBuf[MAX_BUF_LEN];
   switch(code) {
      case '^':
         strncat(dest, "^", MAX_BUF_LEN-1);
         break;
      case 'n':
      case 'N':
         (*colRec).flags=0;
         (*colRec).fgColour=DEF_FG;
         (*colRec).bgColour=DEF_BG;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'H':
         (*colRec).flags|=cfBold;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'h':
         (*colRec).flags&=~cfBold;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'U':
         (*colRec).flags|=cfUnderline;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'u':
         (*colRec).flags&=~cfUnderline;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'L':
         (*colRec).flags|=cfItalics;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'l':
         (*colRec).flags&=~cfItalics;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'K':
         (*colRec).flags|=cfFlash;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'k':
         (*colRec).flags&=~cfFlash;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'I':
         (*colRec).flags|=cfReverse;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'i':
         (*colRec).flags&=~cfReverse;
         writeColRec(tempBuf, colRec);
         strncat(dest, tempBuf, MAX_BUF_LEN-1);
         break;
      case 'd':
         setColour(dest, colRec, 0);
         break;
      case 'D':
         setColour(dest, colRec, 8);
         break;
      case 's':
      case 'S':
         setBackground(dest, colRec, 0);
         break;
      case 'r':
         setColour(dest, colRec, 1);
         break;
      case 'R':
         setColour(dest, colRec, 9);
         break;
      case 'e':
      case 'E':
         setBackground(dest, colRec, 1);
         break;
      case 'g':
         setColour(dest, colRec, 2);
         break;
      case 'G':
         setColour(dest, colRec, 10);
         break;
      case 'f':
      case 'F':
         setBackground(dest, colRec, 2);
         break;
      case 'y':
         setColour(dest, colRec, 3);
         break;
      case 'Y':
         setColour(dest, colRec, 11);
         break;
      case 't':
      case 'T':
         setBackground(dest, colRec, 3);
         break;
      case 'b':
         setColour(dest, colRec, 4);
         break;
      case 'B':
         setColour(dest, colRec, 12);
         break;
      case 'v':
      case 'V':
         setBackground(dest, colRec, 4);
         break;
      case 'p':
         setColour(dest, colRec, 5);
         break;
      case 'P':
         setColour(dest, colRec, 13);
         break;
      case 'o':
      case 'O':
         setBackground(dest, colRec, 5);
         break;
      case 'c':
         setColour(dest, colRec, 6);
         break;
      case 'C':
         setColour(dest, colRec, 14);
         break;
      case 'x':
      case 'X':
         setBackground(dest, colRec, 6);
         break;
      case 'w':
         setColour(dest, colRec, 7);
         break;
      case 'W':
         setColour(dest, colRec, 15);
         break;
      case 'q':
      case 'Q':
         setBackground(dest, colRec, 7);
         break;
   };
}

static void codesToAnsi(char *dest, const char *src) {
   char tempBuf[MAX_BUF_LEN] = "";
   ColRec colRec;
   colRec.flags=0;
   colRec.fgColour=DEF_FG;
   colRec.bgColour=DEF_BG;
   *dest=0;
   if(!src) return;
   while(*src) {
      switch(*src) {
         case '^':
            src++;
            if(*src) addColourCode(dest, &colRec, *src);
            break;
         case '\n':
            colRec.flags=0;
            colRec.fgColour=DEF_FG;
            colRec.bgColour=DEF_BG;
            writeColRec(tempBuf, &colRec);
            strncat(dest, tempBuf, MAX_BUF_LEN-1);
         default:
           tempBuf[0]=*src;
           tempBuf[1]=0;
           strncat(dest, tempBuf, MAX_BUF_LEN-1);
      };
      src++;
   };
}

int cvsprintf(char *buf, const char *fmt, va_list ap) {
   char tempBuf[MAX_BUF_LEN];
   int result = vsprintf(tempBuf, fmt, ap);
   codesToAnsi(buf, tempBuf);
   return result;
}

int csprintf(char *buf, const char *fmt, ...) {
  va_list ap;
  int result;
  va_start(ap, fmt);
  result = cvsprintf(buf, fmt, ap);
  va_end(ap);
  return result;
}

int cvprintf(const char *fmt, va_list ap) {
   char tempBuf[MAX_BUF_LEN];
   int result = cvsprintf(tempBuf, fmt, ap);
   printf("%s", tempBuf);
   return result;
}

void stripCodes(char *dest, const char *src)
{
   int cmode = 0;
   if(!src) return;
   if(!dest) return;
   while (*src) {
      if (*src== '^') {
         if (cmode) {
            *dest++ = '^';
            cmode = 0;
         } else {
            cmode = 1;
         };
      } else {
         if (!cmode) {
            *dest++ = *src;
         } else {
            cmode = 0;
         }
      }
      src++;
   }
   *dest = '\0';
}

int cprintf(const char *fmt, ...) {
   int result;
   va_list ap;
   va_start(ap, fmt);
   result = cvprintf(fmt, ap);
   va_end(ap);
   return result;
}
