/*
 Calisto (c) 1998-1999 Peter Howkins, Matthew Howkins, Simon Howkins

 $Id: privs.c,v 1.4 2000/01/02 15:37:02 peter Exp $

 $Log: privs.c,v $
 Revision 1.4  2000/01/02 15:37:02  peter
 Changed PRIV_AD_GENERAL to PRIV_AD_BASE and added PRIV_SU_BASE

 Revision 1.3  1999/12/20 20:54:09  peter
 Added clearpriv(), privname_to_priv(), and other small changes

 Revision 1.2  1999/12/12 16:28:38  peter
 Added priv a_general

 Revision 1.1  1999/11/29 22:05:17  peter
 Initial revision


 */
static char rcsid[] = "$Id: privs.c,v 1.4 2000/01/02 15:37:02 peter Exp $";

#include "privs.h"
#include "strplus.h"
#include "structs.h"

const rank ranks[] = {
  { "guest",    RANK_GUEST    },
  { "resident", RANK_RESIDENT },
  { "su",       RANK_SU       },
  { "ladmin",   RANK_LA       },
  { "admin",    RANK_ADMIN    },
  { "hadmin",   RANK_HA       },
};
const unsigned rankn = sizeof ranks / sizeof(rank);
const priv privs[] = {
   /* basic guest and ressie privs */
  { "n_base",     PRIV_BASE,       "^n" },
  { "n_talk",     PRIV_TALK,       "^n" },
  { "n_save",     PRIV_SAVE,       "^n" },
   /* su privs */
  { "s_base",     PRIV_SU_BASE,    "^c" },
  { "s_trace",    PRIV_SU_TRACE,   "^c" },
  { "s_bump",     PRIV_SU_BUMP,    "^c" },
   /* admin privs */
  { "a_base",     PRIV_AD_BASE,    "^r" },
  { "a_shutdown", PRIV_AD_SHUTDOWN,"^P" },
  { "a_grant",    PRIV_AD_GRANT,   "^P" },
   /* combination privs */
  { "guest",      C_PRIV_GUEST,    "^n" },
  { "resident",   C_PRIV_RESIDENT, "^n" },
  { "su",         C_PRIV_SU,       "^c" },
  { "la",         C_PRIV_LA,       "^b" },
  { "admin",      C_PRIV_ADMIN,    "^r" },
  { "ha",         C_PRIV_HA,       "^P" }
};
const unsigned privn = sizeof privs / sizeof(priv);

bool haspriv(const character *c, unsigned long priv)
{
  return (c->privs & priv);
}

void setpriv(character *c, unsigned long priv)
{
  c->privs |= priv;
}

void clearpriv(character *c, unsigned long priv)
{
  c->privs &= ~priv;
}

const char *privcol(unsigned long priv)
{
  unsigned i;  

  for (i = 0; i < privn; i++) {
    if (priv == privs[i].priv)
      return privs[i].col;
  }
  return NULL;
}

const priv *privname_to_priv(const char *privname)
{
  unsigned i;

  for (i = 0; i < privn; i++) {
    if (STRIEQ(privname, privs[i].privname))
      return &privs[i];
  }
  return NULL;
}
