/*
 Calisto (c) 1998-1999 Peter Howkins, Matthew Howkins, Simon Howkins

 $Id: privs.h,v 1.2 2000/01/02 15:37:17 peter Exp $

 $Log: privs.h,v $
 Revision 1.2  2000/01/02 15:37:17  peter
 Changed PRIV_AD_GENERAL to PRIV_AD_BASE and added PRIV_SU_BASE

 Revision 1.1  1999/12/21 20:37:26  peter
 Initial revision


 */
#ifndef privs_h
#define privs_h

#include "structs.h"

#define RANK_GUEST    1
#define RANK_RESIDENT 2
#define RANK_SU       3
#define RANK_LA       4
#define RANK_ADMIN    5
#define RANK_HA       6

/* basic guest and ressie privs */
#define PRIV_BASE          (1 << 0)
#define PRIV_TALK          (1 << 1)
#define PRIV_SAVE          (1 << 2)
#define PRIV_BS1           (1 << 3)
#define PRIV_BS2           (1 << 4)
#define PRIV_BS3           (1 << 5)
#define PRIV_BS4           (1 << 6)
#define PRIV_BS5           (1 << 7)
#define PRIV_BS6           (1 << 8)
#define PRIV_BS7           (1 << 9)

/* su privs */
#define PRIV_SU_BASE       (1 << 10)
#define PRIV_SU_BUMP       (1 << 11)
#define PRIV_SU_TRACE      (1 << 12)
#define PRIV_SUS2          (1 << 13)
#define PRIV_SUS3          (1 << 14)
#define PRIV_SUS4          (1 << 15)
#define PRIV_SUS5          (1 << 16)
#define PRIV_SUS6          (1 << 17)
#define PRIV_SUS7          (1 << 18)
#define PRIV_SUS8          (1 << 19)

/* admin privs */
#define PRIV_AD_BASE       (1 << 20)
#define PRIV_AD_SHUTDOWN   (1 << 21)
#define PRIV_AD_GRANT      (1 << 22)
#define PRIV_ADS2          (1 << 23)
#define PRIV_ADS3          (1 << 24)
#define PRIV_ADS4          (1 << 25)
#define PRIV_ADS5          (1 << 26)
#define PRIV_ADS6          (1 << 27)
#define PRIV_ADS7          (1 << 28)
#define PRIV_ADS8          (1 << 29)

/* spare privs */
#define PRIV_S30           (1 << 30)
#define PRIV_S31           (1 << 31)

/* combination privs */
#define C_PRIV_GUEST    (PRIV_TALK | PRIV_BASE)
#define C_PRIV_RESIDENT (C_PRIV_GUEST | PRIV_SAVE)
#define C_PRIV_SU       (C_PRIV_RESIDENT | PRIV_SU_TRACE | PRIV_SU_BUMP | \
                         PRIV_SU_BASE)
#define C_PRIV_LA       (C_PRIV_SU)
#define C_PRIV_ADMIN    (C_PRIV_LA | PRIV_AD_BASE)
#define C_PRIV_HA       (C_PRIV_ADMIN | PRIV_AD_SHUTDOWN | PRIV_AD_GRANT)

typedef struct {
  const char *rankname;
  const int rank;
} rank;

typedef struct {
  const char *privname;
  const unsigned long priv;
  const char *col;
} priv;

extern const priv privs[];
extern const rank ranks[];
extern const unsigned rankn;
extern const unsigned privn;

extern bool haspriv(const character *c, unsigned long priv);
extern void setpriv(character *c, unsigned long priv);
extern void clearpriv(character *c, unsigned long priv);
extern const char *privcol(unsigned long priv);

/* Takes a priv name, and returns the priv number, or 0 if not found */
extern const priv *privname_to_priv(const char *privname);

#endif /* privs_h */
